<li class="dropdown">
  <a href="#" class="dropdown-toggle" data-toggle="dropdown">{{ _('Site') }} <b class="caret"></b></a>
  <ul class="dropdown-menu globaltoc">{{ toctree(maxdepth=1) }}</ul>
</li>
